import pandas as pd
from math import sin, cos, sqrt, asin, radians


def head(header, df, k):
    print(header)
    for row in df[0:k]:
        print(row)
    return None


def load_data(filename):
    """
    Read in a CSV.
    """
    df = pd.read_csv(filename, sep=",")
    df = df.dropna()
    return df


def data_as_list_of_lists(filename):
    return load_data(filename).values.tolist()


def data_header(filename):
    return load_data(filename).columns.tolist()


def filter_numerical(dataset, col_index, min_value, max_value):
    """
    Filters the dataset by a numerical column and return the specified subset
    Parameters: dataset (list-of-lists), col_index (integer), max/min_values (float)
    Returns: a (new) list-of-lists containing all those records in the `dataset`
    with a value in column `col_index` between `min_value` and `max_value`
    """
    subset = []
    for row in dataset:
        if row[col_index] >= min_value and row[col_index] <= max_value:
            subset.append(row)
    return subset


def filter_categorical(dataset, col_index, categories):
    """
    Filters the dataset by a categorical column and return the specified subset
    Parameters: dataset (list-of-lists), col_index (integer), categories (list)
    Returns: a (new) list-of-lists containing all those records in the `dataset`
    with a value in column `col_index` in the `categories` list
    """
    subset = []
    for row in dataset:
        if row[col_index] in categories:
            subset.append(row)
    return subset


def distance(loc1, loc2):
    """
    Calculate the distance between two locations using the Haversine formula.

    Parameters
    ----------

        loc1, loc2 (2-tuple of floats)
           Locations encoded as (latitude, longitude) pairs

    Returns
    --------

        dist (float)
            The distance between the two locations, in miles

    """

    lat1, long1 = loc1
    lat2, long2 = loc2

    delta_lat = radians(lat2 - lat1)
    delta_long = radians(long2 - long1)

    lat1 = radians(lat1)
    lat2 = radians(lat2)

    RADIUS = 3959.87433
    a = sin(delta_lat / 2) ** 2 + cos(lat1) * cos(lat2) * sin(delta_long / 2) ** 2

    return RADIUS * 2 * asin(sqrt(a))


if __name__ == "__main__":
    # you would select different subsets here
    # subset = data_from_file("nvcensus-data-v2.csv", "1920", "PAHRUMP PRECINCT")

    census_df = pd.read_csv(
        "nvcensus-data-v2.csv", sep="|", na_values=[r"\N"], low_memory=False
    )
    census_df = census_df[
        ["Year", "Subunit", "FName", "Surname", "Age", "Sex", "Color", "Profession"]
    ]

    census_df = census_df[census_df.Year == 1920]
    # census_df = census_df[census_df.Subunit == "PAHRUMP PRECINCT"]
    census_df.to_csv("census.csv", sep=",", header=True, index=False)

    bikes_df = pd.read_csv("201908-bluebikes-tripdata", sep=",")

    bikes_df["start_location"] = list(
        zip(bikes_df["start station latitude"], bikes_df["start station longitude"])
    )
    bikes_df["end_location"] = list(
        zip(bikes_df["end station latitude"], bikes_df["end station longitude"])
    )

    bikes_df = bikes_df.rename(
        columns={
            "tripduration": "trip_duration",
            "starttime": "start_time",
            "stoptime": "end_time",
            "usertype": "user_type",
            "birth year": "birth_year",
            "start station name": "start_name",
            "end station name": "end_name",
        }
    )

    bikes_df = bikes_df.drop(
        columns=[
            "start station latitude",
            "start station longitude",
            "end station latitude",
            "end station latitude",
        ]
    )

    bikes_df = bikes_df[
        [
            "bikeid",
            "start_name",
            "end_name",
            "start_location",
            "end_location",
            "trip_duration",
            "user_type",
            "birth_year",
            "gender",
        ]
    ]

    bikes_df["distance"] = bikes_df[["start_location", "end_location"]].apply(
        lambda x: distance(*x), axis=1
    )

    bikes_df = bikes_df.drop(columns=["start_location", "end_location"])

    bikes_df = bikes_df[bikes_df.bikeid.isin(list(range(5550, 6000)))]
    bikes_df.to_csv("bluebikes.csv", sep=",", header=True, index=False)
